import matplotlib.pyplot as plt
import numpy as np                
import scipy.stats as stat
from netCDF4 import Dataset

# set general text options for plotting
plt.rc('text'           , usetex=True)
plt.rc('font'           , size=12)
plt.rc('legend'         , fontsize=12)
plt.rc('text.latex'     , preamble=r'\usepackage{cmbright}')

#--------------------------------------------------------------------------
def summaryStats(samples):
        statstring   =  ' '+                                                   \
                        str('%.2f' % round(stat.scoreatpercentile(samples,50),2))+' ['  \
                       +str('%.2f' % round(stat.scoreatpercentile(samples,5),2)) +' - ' \
                       +str('%.2f' % round(stat.scoreatpercentile(samples,95),2))+'] '
        return(statstring)

#--------------------------------------------------------------------------

nsamp = int(5e6)
from95tostd = stat.norm.ppf(0.95)
almost_black            = '#262626'

#----------------------------------------------------------------------
# Load HadCRUT temperatures from file. The file as downloaded from CRU has been pre-processes with CDO:
# cdo -a settunits,years -selyear,1850/2016 -fldmean -yearmean HadCRUT.4.5.0.0.median.nc HadCRUT.4.5.0.0.median_means.nc
#
f           = Dataset('HadCRUT.4.5.0.0.median_means.nc', mode='r')
HadCRU_year = f.variables['time'][:]
HadCRU_t    = f.variables['temperature_anomaly'][:,0,0]

# Adjust the temperature relative to 1850-1899:
HadCRU_t    = HadCRU_t - np.mean(HadCRU_t[np.where((HadCRU_year >= 1850) & (HadCRU_year <= 1899))])

# Calculate temperature of the first reference period:
HadCRU_t1    = np.mean(HadCRU_t[np.where((HadCRU_year >= 1859) & (HadCRU_year <= 1882))])    

# Calculate temperature of the second reference period:
HadCRU_t2    = np.mean(HadCRU_t[np.where((HadCRU_year >= 2005) & (HadCRU_year <= 2015))])    

# Calculate temperature change to the second reference period:
HadCRU_dt   = HadCRU_t2 - HadCRU_t1

# Standard deviation on temperature change from uncertainty and variability (Lewis and Curry 2014):
HadCRU_sd   = 0.08

# Randomly sample temperatures:
delta_t     = np.random.normal(loc=HadCRU_dt, scale=HadCRU_sd, size=nsamp)

#----------------------------------------------------------------------
# Radiation imbalance for 2005-2015 from Johnson et al. (2016), doi:10.1038/nclimate3043:
Q2_mean = 0.71
Q2_sd   = 0.10/from95tostd

# Imbalance for 1859-1882 from Lewis and Curry (2014), based on GCM:
Q1_mean = 0.15
Q1_sd   = 0.075

# Ramdomly sample imbalance and imbalance difference:
imbalance = np.random.normal(loc=Q2_mean, scale=Q2_sd, size=nsamp)
delta_imbalance = imbalance - np.random.normal(loc=Q1_mean, scale=Q1_sd, size=nsamp)

#----------------------------------------------------------------------
# Load AR5 forcings from file:
F_year, F_co2, F_ghgother, F_o3trop, F_o3strat, F_aero, F_luc, F_h2ostrat, F_bcsnow, F_contrails, F_solar, F_volc = \
  np.loadtxt('WG1_AR5_Annex_II_sheet_1-2_forcings.txt', skiprows=1, unpack=True)

# Calculate total forcing, and assume volcanoes are on average zero:
F_total = F_co2 + F_ghgother + F_o3trop + F_o3strat + F_aero + F_luc + F_h2ostrat + F_bcsnow + F_contrails + F_solar + F_volc
F_total = F_total - np.mean(F_volc)

# Identify indices (i1 and i2) for the reference periods. For the later 
# reference period of 2005-2015 we choose the values in the year 2010 as 
# the average forcing representative of the period.
i1 = np.where((F_year >= 1859) & (F_year <= 1882))
i2 = np.where(F_year == 2010)

# Forcing from a doubling of CO2 that is consistent with :
F2x_mean               =   3.71

# Mean change in forcings between reference periods:
delta_F_ghg_mean       =   np.mean(F_co2[i2])-np.mean(F_co2[i1]) \
                         + np.mean(F_ghgother[i2])-np.mean(F_ghgother[i1])
delta_F_o3_mean        =   np.mean(F_o3trop[i2])-np.mean(F_o3trop[i1]) \
                         + np.mean(F_o3strat[i2])-np.mean(F_o3strat[i1])
delta_F_aero_mean      =   np.mean(F_aero[i2])-np.mean(F_aero[i1])
F_aero_mean            =   np.mean(F_aero[i2])
delta_F_luc_mean       =   np.mean(F_luc[i2])-np.mean(F_luc[i1])
delta_F_h2ostrat_mean  =   np.mean(F_h2ostrat[i2])-np.mean(F_h2ostrat[i1])
delta_F_bcsnow_mean    =   np.mean(F_bcsnow[i2])-np.mean(F_bcsnow[i1])
delta_F_contrails_mean =   np.mean(F_contrails[i2])-np.mean(F_contrails[i1])
delta_F_nat_mean       =   np.mean(F_solar[i2])-np.mean(F_solar[i1]) \
                         + np.mean(F_volc[i2])-np.mean(F_volc[i1])

delta_F_mean           =   delta_F_ghg_mean + delta_F_o3_mean + delta_F_aero_mean \
                         + delta_F_luc_mean + delta_F_h2ostrat_mean + delta_F_bcsnow_mean \
                         + delta_F_contrails_mean + delta_F_nat_mean 

#----------------------------------------------------------------------
# Estimate forcing from short lived climate forcers, CH4, NOx and CO
# Estimates of emissions-based forcing including impacts on other GHG concentrations from
# IPCC AR5, Table 8.SM.6 (supplementary material for chapter 8)
F_CH4_mean             =  0.970
F_NOx_mean             = -0.151
F_CO_mean              =  0.234
    
# Fraction of anthropogenic methane emissions due to fossil fuel consumption (IPCC AR5 Table 6.8)
fraction_CH4_mean      = 96./331.
    
#----------------------------------------------------------------------
# Define standard deviations on all forcings. These are taken from
# AR5 wherever possible.

# Well mixed greenhouse gases: 
# "The RF of WMGHG is 2.83 (2.54 to 3.12)"
ghg_sd = (3.12 - 2.54)/(2*from95tostd)

# Forcing from CO2-doubling alone, 20 percent as in Lewis and Curry (2014):
F2x_sd = ghg_sd*F2x_mean/delta_F_ghg_mean

# Ozone:
# " total RF estimated from modelled ozone changes is 0.35 (0.15 to 0.55) W m–2, 
#   with RF due to tropospheric ozone changes of 0.40 (0.20 to 0.60) W m–2 and 
#   due to stratospheric ozone changes of –0.05 (–0.15 to +0.05) W m–2" 
o3_sd = (0.55 - 0.14)/(2*from95tostd)
    
# Aerosols: 
# "is estimated as ERF of –0.9 (–1.9 to –0.1) W m–2."
aero_sd = (1.9 - 0.1)/(2*from95tostd)
    
# Land use change:  
# "leads to an RF of –0.15 ± 0.10 W m–2", here we assume this means 5-95 percentiles
luc_sd = 0.1/from95tostd 

# Stratospheric water vapor:
# "is 0.07 (0.02 to 0.12) W m–2" 
h2ostrat_sd = (0.12 - 0.02)/(2*from95tostd)

# Black carbon (BC) on snow and ice:
# "is 0.04 (0.02 to 0.09) W m–2. "
bcsnow_sd = (0.09 - 0.02)/(2*from95tostd)

# Contrails: no uncertainty range found.
contrails_sd = 0.0

# Natural forcings (volcanic and solar): no uncertainty range found.
nat_sd = 0.0

# Short lived climate forcers, IPCC AR5 tables 8.SM.7 and 6.8:
F_CH4_sd             =  F_CH4_mean*0.17/from95tostd
F_NOx_sd             = -F_NOx_mean*1.20/from95tostd
F_CO_sd              =  F_CO_mean*0.24/from95tostd
fraction_CH4_sd      =  10./331./from95tostd

#----------------------------------------------------------------------
# Randomly sample forcings as Gaussian distributions:

delta_F_aero    =  np.random.normal(loc=delta_F_aero_mean, scale=aero_sd, size=nsamp)
total_F_aero    =  delta_F_aero + (F_aero_mean-delta_F_aero_mean)
delta_F_ghg     =  np.random.normal(loc=delta_F_ghg_mean, scale=ghg_sd, size=nsamp)
delta_F_nonaero =  np.random.normal(loc=delta_F_o3_mean, scale=o3_sd, size=nsamp) \
                  +np.random.normal(loc=delta_F_luc_mean, scale=luc_sd, size=nsamp) \
                  +np.random.normal(loc=delta_F_h2ostrat_mean, scale=h2ostrat_sd, size=nsamp) \
                  +np.random.normal(loc=delta_F_bcsnow_mean, scale=bcsnow_sd, size=nsamp) \
                  +delta_F_contrails_mean + delta_F_nat_mean
delta_F         =  delta_F_aero + delta_F_ghg + delta_F_nonaero

# Make errors in forcing from CO2 doubling correlate with errors in greenhouse gas forcing:
F2x             = (delta_F_ghg-delta_F_ghg_mean)*F2x_sd/ghg_sd + F2x_mean

# Short lived climate forcers:
F_CH4           = np.random.normal(loc=F_CH4_mean, scale=F_CH4_sd, size=nsamp)
F_NOx           = np.random.normal(loc=F_NOx_mean, scale=F_NOx_sd, size=nsamp)
F_CO            = np.random.normal(loc=F_CO_mean, scale=F_CO_sd, size=nsamp)
fraction_CH4    = np.random.normal(loc=fraction_CH4_mean, scale=fraction_CH4_sd, size=nsamp)
total_F_slcf    = fraction_CH4*F_CH4 + F_NOx + F_CO    

print(' ')
print('----------------------------------------')
print('GHG forcing:         '+summaryStats(delta_F_ghg))
print('Aerosol forcing:    ' +summaryStats(delta_F_aero))
print('Non-aerosol forcing: '+summaryStats(delta_F_nonaero))
print('----------------------------------------')
print('Total forcing change:'+summaryStats(delta_F))
print(' ')

print('----------------------------------------')
print('Aerosol forcing uncertainty is '\
      + str('%.2f' % round(100.*np.std(total_F_aero)**2/(np.std(delta_F - delta_imbalance)**2),2)) \
      +'% of the variance in the denominator of ECS')

print('Short lived climate forcers:'+summaryStats(total_F_slcf))
    
# Calculate correction for the greenhouse gas increase from 2010 to 2016 based on the rate after year 2000:
id_rate      = np.where(F_year >= 2000)
F_ghgrate    = stat.linregress(F_year[id_rate],F_co2[id_rate]+F_ghgother[id_rate])
F_correction = F_ghgrate.slope*(2016-2010)
print('Correction of forcing from 2010 to present:  '+str('%.2f' % round(F_correction,2)))
print(' ')

#----------------------------------------------------------------------
# Calculate ECS and TCR from samples:

TCR  = F2x*delta_t/delta_F
ECS  = F2x*delta_t/(delta_F-delta_imbalance)

# Set infinite values to not any number:
id_inf      = np.where((TCR<0.0) | (ECS<0.0))
TCR[id_inf] = float('nan')
ECS[id_inf] = float('nan')
id          = np.where(np.isfinite(ECS))

print('----------------------')
print('Climate sensitivities:')
print('TCR:'+summaryStats(TCR[id]))
print('ECS:'+summaryStats(ECS[id]))
print('----------------------')
print(' ')

#----------------------------------------------------------------------
# We define 1850-1899 as pre-industrial reference for total committed warming, but
# how much warmer was this period then expectation for 1750? Observations do not date
# back this long, but the estimated forcings allow us to estimate the warming:
i3 = np.where((F_year >= 1850) & (F_year <= 1899))
print('Mean forcing in 1850-1899 wrt. 1750 (Wm-2) : '+str('%.2f' % round(np.mean(F_total[i3]),2)))
print('Transient response (K)                     :'+summaryStats(np.mean(F_total[i3])*TCR[id]/F2x[id]))
print(' ')

#----------------------------------------------------------------------
# Go through commitment scenarios

# Specific climate sensitivity per unit forcing:
s = ECS/F2x

# a) Equilibrium with constant forcing:
equil_constant_forcing = delta_t + HadCRU_t1 + s*(imbalance + F_correction)

print('a) Equilibrium with constant-forcing commitment:')
print('Median: '+summaryStats(equil_constant_forcing[id]))
print('Chance of exceeding 1.5K: '+str(round(100.-stat.percentileofscore(equil_constant_forcing[id],1.5),1)) + '%')
print('Chance of exceeding 2.0K: '+str(round(100.-stat.percentileofscore(equil_constant_forcing[id],2.0),1)) + '%')
print(' ')

# b) Equilibrium with removed aerosol forcing:
equil_aerosol_forcing = delta_t + HadCRU_t1 + s*(imbalance - (total_F_aero) + F_correction) 

print('b) Equilibrium with removed aerosol forcing commitment:')
print('Median: '+summaryStats(equil_aerosol_forcing[id]))
print('Chance of exceeding 1.5K: '+str(round(100.-stat.percentileofscore(equil_aerosol_forcing[id],1.5),1)) + '%')
print('Chance of exceeding 2.0K: '+str(round(100.-stat.percentileofscore(equil_aerosol_forcing[id],2.0),1)) + '%')
print(' ')

# c) Equilibrium with removed aerosol and SLCF forcing:
equil_aeroslcf_forcing = delta_t + HadCRU_t1 + s*(imbalance - (total_F_aero+total_F_slcf) + F_correction) 

print('c) Equilibrium with removed aerosol and SLCF forcing commitment:')
print('Median: '+summaryStats(equil_aeroslcf_forcing[id]))
print('Chance of exceeding 1.5K: '+str(round(100.-stat.percentileofscore(equil_aeroslcf_forcing[id],1.5),1)) + '%')
print('Chance of exceeding 2.0K: '+str(round(100.-stat.percentileofscore(equil_aeroslcf_forcing[id],2.0),1)) + '%')
print(' ')
 
# Centennial response fractions, by default these are all scaled with TCR to ECS:
alpha_l = TCR/ECS  # Remnant forcing 
alpha_s = alpha_l  # Abruptly applied forcing
alpha_i = alpha_l  # Intermediate case

# d) Centennial commitment to remnant and aerosol forcings:
centennial_aeroslcf_forcing = delta_t + HadCRU_t1 + s*(alpha_l*imbalance - alpha_s*(total_F_aero+total_F_slcf) + alpha_i*F_correction)

print('d) Centennial commitment with removed aerosol and SLCF forcing:')
print('Median: '+summaryStats(centennial_aeroslcf_forcing[id]))
print('Chance of exceeding 1.5K: '+str(round(100.-stat.percentileofscore(centennial_aeroslcf_forcing[id],1.5),1)) + '%')
print('Chance of exceeding 2.0K: '+str(round(100.-stat.percentileofscore(centennial_aeroslcf_forcing[id],2.0),1)) + '%')
print(' ')

# e) Centennial commitment to remnant and aerosol and SLCF forcings, and carbon uptake:
centennial_aeroslcf_carbon = delta_t + HadCRU_t1 + s*(alpha_l*(imbalance-np.mean(imbalance)) - alpha_s*(total_F_aero+total_F_slcf) \
                                     + alpha_i*F_correction)

print('e) Centennial commitment with carbon uptake cancelling remnant warming:')
print('Median: '+summaryStats(centennial_aeroslcf_carbon[id]))
print('Chance of exceeding 1.5K: '+str(round(100.-stat.percentileofscore(centennial_aeroslcf_carbon[id],1.5),1)) + '%')
print('Chance of exceeding 2.0K: '+str(round(100.-stat.percentileofscore(centennial_aeroslcf_carbon[id],2.0),1)) + '%')
print(' ')

#----------------------------------------------------------------------
#----------------------------------------------------------------------
# Plots
#----------------------------------------------------------------------
#----------------------------------------------------------------------
    
colora = 'black'
colorb = 'red'
colorc = 'purple'
colord = 'deepskyblue'
colore = 'orange'

# Calculate histograms:
bins        = np.arange(0,10,0.02)
bin_centers = (bins[1:]+bins[:-1])/2.
    
pTCR, hTCR = np.histogram(TCR[id],bins)
pECS, hECS = np.histogram(ECS[id],bins)
ratio      = TCR[id]/ECS[id]
pratio, hratio = np.histogram(ratio,np.arange(0,1,0.005))
    
#----------------------------------------------------------------------
# Figure 1, probabilities of ECS and TCR.
    
fig, axes = plt.subplots(2,1, figsize=(4,5))

i = 0

axes[i].plot(bin_centers, pTCR, color='black', label='TCR')
axes[i].plot(bin_centers, pECS, color='black',linestyle='--', label='ECS')

axes[i].set_xlim([0, 6])
xmax = max(axes[i].get_xlim())
ymax = max(axes[i].get_ylim())
axes[i].set_xlabel('Temperature [K]')

axes[i].text(0,ymax,'a)', color='black', va='top', ha='left')
axes[i].text(xmax*0.8,ymax*0.61,'ECS: '+summaryStats(ECS[id]), color='black', va='center', ha='right', fontsize=10)
axes[i].text(xmax*0.8,ymax*0.7, 'TCR: '+summaryStats(TCR[id]), color='black', va='center', ha='right', fontsize=10)
axes[i].plot([xmax*0.83,xmax*0.9],[ymax*0.61,ymax*0.61], color='black',linestyle='--')
axes[i].plot([xmax*0.83,xmax*0.9],[ymax*0.7,ymax*0.7], color='black')
    
spines_to_remove        = ['top', 'right', 'left'] 
for spine in spines_to_remove:
    axes[i].spines[spine].set_visible(False)

axes[i].set_yticks([])
axes[i].yaxis.set_ticks_position('none')
axes[i].xaxis.set_ticks_position('bottom')

spines_to_keep = [ 'bottom']     
for spine in spines_to_keep:
    axes[i].spines[spine].set_linewidth(0.5)
    axes[i].spines[spine].set_color(almost_black)

i = 1

axes[i].plot((hratio[1:]+hratio[:-1])/2.0, pratio, color='black')

axes[i].set_xlim([0, 1])
xmax = max(axes[i].get_xlim())
ymax = max(axes[i].get_ylim())
axes[i].set_xlabel('TCR/ECS')

axes[i].text(0,ymax,'b)', color='black', va='top', ha='left')
axes[i].text(xmax*0.1,ymax*0.7, 'TCR/ECS: '+summaryStats(ratio), color='black', va='top', ha='left', fontsize=10)

spines_to_remove        = ['top', 'right', 'left'] 
for spine in spines_to_remove:
    axes[i].spines[spine].set_visible(False)

axes[i].set_yticks([])
axes[i].yaxis.set_ticks_position('none')
axes[i].xaxis.set_ticks_position('bottom')

spines_to_keep = [ 'bottom']     
for spine in spines_to_keep:
    axes[i].spines[spine].set_linewidth(0.5)
    axes[i].spines[spine].set_color(almost_black)

plt.tight_layout()
plt.savefig('Figure_1_ECS_TCR.pdf', dpi=600)
plt.close()

#----------------------------------------------------------------------
# Figure 2, Observed temperature and committed warmings.

fig, axes = plt.subplots(figsize=(7,4))
xmin = 1840
xmax = 2090
axes.axis([xmin, xmax, -0.3, 4.1])   

bcolor = almost_black 

axes.plot(HadCRU_year,HadCRU_t, color='gray')
axes.plot((2005,2015),(HadCRU_t2,HadCRU_t2), color=bcolor, lw=1.5)
axes.text(2016,HadCRU_t2,str(round(HadCRU_t2,2)), color=almost_black, va='center', ha='left', fontsize=11)
axes.plot((1859,1882),(HadCRU_t1,HadCRU_t1), color=bcolor, lw=1.5)
axes.text(1865,HadCRU_t1,str(round(HadCRU_t1,2)), color=almost_black, va='bottom', ha='right', fontsize=11)
axes.plot((2030, 2080), (1.5, 1.5),'--',color='gray')
axes.plot((2030, 2080), (2, 2),'--',color='gray')

# a) Equilibrium with constant forcing:
x=2035
axes.scatter(x,   stat.scoreatpercentile(equil_constant_forcing[id],50), color=colora, zorder=10)    
axes.plot((x,x), (stat.scoreatpercentile(equil_constant_forcing[id],5), \
                  stat.scoreatpercentile(equil_constant_forcing[id],95)), color=colora)
axes.text(x-2,stat.scoreatpercentile(equil_constant_forcing[id],50),'a',va='center',ha='right',fontsize=11, color=colora)

# b) Equilibrium with removed aerosol forcing:
x=2045
axes.scatter(x,   stat.scoreatpercentile(equil_aerosol_forcing[id],50), color=colorb, zorder=10)  
axes.plot((x,x), (stat.scoreatpercentile(equil_aerosol_forcing[id],5), \
                  stat.scoreatpercentile(equil_aerosol_forcing[id],95)), color=colorb)
axes.text(x-2,stat.scoreatpercentile(equil_aerosol_forcing[id],50),'b',va='center',ha='right',fontsize=11, color=colorb)

# c) Equilibrium with removed aerosol and SLCF forcing:
x=2055
axes.scatter(x,   stat.scoreatpercentile(equil_aeroslcf_forcing[id],50), color=colorc, zorder=10)  
axes.plot((x,x), (stat.scoreatpercentile(equil_aeroslcf_forcing[id],5), \
                  stat.scoreatpercentile(equil_aeroslcf_forcing[id],95)), color=colorc)
axes.text(x-2,stat.scoreatpercentile(equil_aeroslcf_forcing[id],50)-0.05,'c',va='center',ha='right',fontsize=11, color=colorc)

# d) Centennial commitment to remnant and aerosol and SLCF forcings:
x=2065
axes.scatter(x,   stat.scoreatpercentile(centennial_aeroslcf_forcing[id],50), color=colord, zorder=10)  
axes.plot((x,x), (stat.scoreatpercentile(centennial_aeroslcf_forcing[id],5), \
                  stat.scoreatpercentile(centennial_aeroslcf_forcing[id],95)), color=colord)
axes.text(x-2,stat.scoreatpercentile(centennial_aeroslcf_forcing[id],50),'d',va='center',ha='right',fontsize=11, color=colord)

# e) Centennial commitment to remnant and aerosol and SLCF forcings, and carbon uptake:
x=2075
axes.scatter(x,   stat.scoreatpercentile(centennial_aeroslcf_carbon[id],50), color=colore, zorder=10)  
axes.plot((x,x), (stat.scoreatpercentile(centennial_aeroslcf_carbon[id],5), \
                  stat.scoreatpercentile(centennial_aeroslcf_carbon[id],95)), color=colore)
axes.text(x-2,stat.scoreatpercentile(centennial_aeroslcf_carbon[id],50),'e',va='center',ha='right',fontsize=11, color=colore) 

commit_text1 = 'Equilibrium with constant forcing (a): \n'\
               '.. and removed aerosol cooling (b): \n'\
               '.. and removed SLCFs (c): \n'\
               'Centennial committed warming (d): \n'\
               '.. with carbon uptake (e): '
commit_text2 = summaryStats(equil_constant_forcing[id])+'\n'\
              +summaryStats(equil_aerosol_forcing[id])+'\n'\
              +summaryStats(equil_aeroslcf_forcing[id])+'\n'\
              +summaryStats(centennial_aeroslcf_forcing[id])+'\n'\
              +summaryStats(centennial_aeroslcf_carbon[id])

axes.text(1960,3.5,commit_text1, color=almost_black, fontsize=11, va='top', ha='right')
axes.text(1963,3.5,commit_text2, color=almost_black, fontsize=11, va='top', ha='left')

plt.ylabel('Temperature change relative to 1850-1899 (K)')

plt.xticks((1850,1900,1950,2000,max(HadCRU_year)))
plt.yticks(np.arange(0.0,5.0,1.0))

axes.spines['bottom'].set_position('zero')

spines_to_remove        = ['top', 'left'] 
for spine in spines_to_remove:
    axes.spines[spine].set_visible(False)

axes.yaxis.set_ticks_position('right')
axes.xaxis.set_ticks_position('bottom')
axes.yaxis.set_label_position("right")

spines_to_keep = ['right', 'bottom']     
for spine in spines_to_keep:
    axes.spines[spine].set_linewidth(0.5)
    axes.spines[spine].set_color(almost_black)

for loc, s in axes.spines.items():
    s.set_capstyle('projecting')

for ticks in axes.xaxis.get_ticklines() + axes.yaxis.get_ticklines():
    ticks.set_color(almost_black)
        
plt.tight_layout()
plt.savefig('Figure_2_committed_warming.pdf', dpi=600)
plt.close()

#----------------------------------------------------------------------
# Figure 3, Commitment versus TCR.

xmax=3.5
ymax=3.0

fig, axes = plt.subplots(figsize=(4,3.5))
    
bins = np.arange(0.0,xmax+0.05,0.05)
TCR_hist, bins             = np.histogram(TCR[np.where(np.isfinite(TCR))],bins)
TCR_pos_aero_hist, bins    = np.histogram(TCR[np.where(np.isfinite(TCR) & (delta_F_aero >  0.0))], bins)
TCR_strong_aero_hist, bins = np.histogram(TCR[np.where(np.isfinite(TCR) & (delta_F_aero < -1.0))], bins)

bin_centers = (bins[1:]+bins[:-1])/2.
bin_width   = bins[1]-bins[0]
n           = sum(TCR_hist)
axes.fill_between(bin_centers, 0, TCR_hist/bin_width/n, color='gray', alpha=0.3)
axes.fill_between(bin_centers, 0, TCR_pos_aero_hist/bin_width/n, color='darkcyan', alpha=0.5)
axes.fill_between(bin_centers, 0, TCR_strong_aero_hist/bin_width/n, color='saddlebrown', alpha=0.5)

axes.plot((1.3, 2.2), (1.5, 1.5),color='gray')
axes.text(1.3-0.05,1.5,'1.5', color='gray', va='center', ha='right', fontsize=10)
axes.plot((1.8, 2.9), (2, 2),color='gray')
axes.text(1.8-0.05,2,'2.0', color='gray', va='center', ha='right', fontsize=10)
axes.plot((0.6, 1.15),(HadCRU_t2,HadCRU_t2), color='gray')
axes.text(0.6-0.05,HadCRU_t2,str(round(HadCRU_t2,2)), color='gray', va='center', ha='right', fontsize=10)

# d) Centennial commitment to remnant, aerosol and SLCF forcings:
y,x,n   = stat.binned_statistic(TCR[id],centennial_aeroslcf_forcing[id], np.median, bins=np.arange(0.5,3,0.1))
std,x,n = stat.binned_statistic(TCR[id],centennial_aeroslcf_forcing[id], np.std, bins=x)
x       = (x[1:]+x[:-1])/2.0
axes.plot(x,y, color=colord)
 
# e) Centennial commitment to remnant, aerosol and SLCF forcings, and carbon uptake:
y,x,n   = stat.binned_statistic(TCR[id],centennial_aeroslcf_carbon[id], np.median, bins=np.arange(0.6,3.1,0.1))
std,x,n = stat.binned_statistic(TCR[id],centennial_aeroslcf_carbon[id], np.std, bins=x)
x       = (x[1:]+x[:-1])/2.0
axes.plot(x,y, color=colore)

axes.text(1.3,2.7,'Centennial committed warming (d)',color=colord,ha='center', fontsize=10)
axes.text(1.3,2.5,'.. with carbon uptake (e)',color=colore,ha='center', fontsize=10)
    
axes.text(2.8,0.64,r'Probability of TCR', color='gray', ha='center', fontsize=10)
axes.text(2.8,0.41,r'$F_{aero} >  0.0$ Wm$^{-2}$', color='darkcyan', ha='center', fontsize=10)
axes.text(2.8,0.23,r'$F_{aero} < -1.0$ Wm$^{-2}$', color='saddlebrown', ha='center', fontsize=10)

plt.xlim((0,xmax))
plt.ylim((0,ymax))
plt.yticks(np.arange(0.0,4.0,1.0))
plt.xlabel('Transient Climate Response (TCR) [K]')
plt.ylabel('Centennial committed warming [K]')

spines_to_remove        = ['top', 'right'] 
for spine in spines_to_remove:
    axes.spines[spine].set_visible(False)

axes.yaxis.set_ticks_position('left')
axes.xaxis.set_ticks_position('bottom')

spines_to_keep = [ 'bottom', 'left']     
for spine in spines_to_keep:
    axes.spines[spine].set_linewidth(0.5)
    axes.spines[spine].set_color(almost_black)

for ticks in axes.xaxis.get_ticklines() + axes.yaxis.get_ticklines():
    ticks.set_color(almost_black)

plt.tight_layout()
plt.savefig('Figure_3_CW_vs_TCR.pdf', dpi=600)
plt.close()
