#--------------------------------------------------------------------------
# The two-layer model as a function. The two first arguments are mandatory
# in twolayermodel, whereas the rest is optional:
#
# forcing_years: a list of the years in the input_forcing
# input_forcing: the yearly forcing value
# ECS:           the equilibrium climate sensitivity
# gamma:         deep ocean heat uptake efficiency
# T_ml0:         the mixed-layer initial temperature
# T_deep0:       the deep ocean initial temperature
# b:             state-dependency of feedback parameter (typically -0.1 to 0.1)
# efficacy:      ocean heat uptake efficacy parameter (typically 1.0 to 2.0)
#
#--------------------------------------------------------------------------

import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as stats
import math
from scipy.io import netcdf

# set general text options for plotting
plt.rc('text'           , usetex=True)
plt.rc('font'           , size=12)
plt.rc('legend'         , fontsize=12)
plt.rc('text.latex'     , preamble=r'\usepackage{cmbright}')

almost_black = '#262626'

# Define time parameters in seconds:
hour       = 3600
day        = 24*hour
month      = 30*day
year       = 12*month

# Forcing from a doubling of CO2:
f2x        = 3.7

def twolayermodel(forcing_years, input_forcing, ECS=3.0, gamma=1.0, T_ml0=0.0, T_deep0=0.0, b=0.0, efficacy=1.0):

    result = {}

    nyears     = np.size(forcing_years)
    delta_time = month
    nstep      = round(nyears*year/delta_time)
    time       = delta_time*np.arange(0,nstep,dtype=float) + min(forcing_years)*year
    timeyears  = time/year
    forcing    = np.interp(timeyears,forcing_years,input_forcing)

    # Parameters:
    density    = 1000.
    c_w        = 4181.
    C_ml       = 75*0.7*density*c_w
    C_deep     = 1000*0.7*density*c_w
    lambda_0   = -f2x/ECS

    # Initialize state variables:
    T_ml       = np.zeros(nstep)
    T_deep     = np.zeros(nstep)
    T_ml[0]    = T_ml0
    T_deep[0]  = T_deep0
    imbalance  = np.zeros(nstep)
    
    # Integrate:
    imbalance[0] = np.nan 
    for t in range(0, nstep-1):
        T_ml[t+1]      = T_ml[t] + (forcing[t]+(lambda_0+b*T_ml[t])*T_ml[t]-efficacy*gamma*(T_ml[t]-T_deep[t]))*delta_time/C_ml
        T_deep[t+1]    = T_deep[t] + gamma*(T_ml[t]-T_deep[t])*delta_time/C_deep
        imbalance[t+1] = (C_ml*(T_ml[t+1]-T_ml[t]) + C_deep*(T_deep[t+1]-T_deep[t]))/delta_time
    
    # Output result and settings for this run:
    result['time'] = timeyears
    result['forcing'] = forcing
    result['T_ml'] = T_ml
    result['T_deep'] = T_deep
    result['imbalance'] = imbalance

    result['ECS'] = ECS
    result['C_ml'] = C_ml
    result['C_deep'] = C_deep
    result['gamma'] = gamma
    result['lambda_0'] = lambda_0
    result['b'] = b
    result['efficacy'] = efficacy
        
    return result

#--------------------------------------------------------------------------

# Time axis for full experiment:

nyears     = 260+90
timeyears  = np.arange(0,nyears,1) + 1750.
Forcing    = np.zeros(nyears)

#--------------------------------------------------------------------------

# Forcing historical from IPCC AR5:

F_year, F_co2, F_ghgother, F_o3trop, F_o3strat, F_aero, F_luc, F_h2ostrat, F_bcsnow, F_contrails, F_solar, F_volc = \
  np.loadtxt('WG1_AR5_Annex_II_sheet_1-2_forcings.txt', skiprows=1, unpack=True)

# Set mean volcanic forcing to zero:
F_volc = F_volc - np.mean(F_volc)
F_total = F_co2 + F_ghgother + F_o3trop + F_o3strat + F_aero + F_luc + F_h2ostrat + F_bcsnow + F_contrails + F_solar + F_volc 

Forcing[0:np.size(F_total)] = F_total

#--------------------------------------------------------------------------

# Extend using commitment scenario 'd' wherein all aerosol forcing and SLCFs is
# removed abruptly, plus correction for greenhouse gas forcing increase till 2016:

id1 = np.where((timeyears >= 2010) & (timeyears < 2016))
Forcing[id1] = F_total[-1] + 0.03*(timeyears[id1]-min(timeyears[id1]))
id2 = np.where(timeyears >= 2016)
Forcing[id2] = max(Forcing[id1]) - F_aero[-1] -0.36

#--------------------------------------------------------------------------

# Two-layer model:

exp1     = twolayermodel(timeyears, Forcing, ECS=1.78)
exp2     = twolayermodel(timeyears, Forcing, ECS=1.95, efficacy=1.28)
exp3     = twolayermodel(timeyears, Forcing, ECS=2.39, efficacy=1.82)
idrmodel = np.where((exp1['time'] >= 1850) & (exp1['time'] < 1900))

#--------------------------------------------------------------------------

# Observed temperature:

f_HadCRU    = netcdf.netcdf_file('HadCRUT.4.5.0.0.median_means.nc')
HadCRU_year = f_HadCRU.variables['time'][:]
HadCRU_t    = f_HadCRU.variables['temperature_anomaly'][:,0,0]
idrHad      = np.where((HadCRU_year >= 1850) & (HadCRU_year < 1899))

#--------------------------------------------------------------------------

# Plot

fig, axes = plt.subplots(1,1, figsize=(8,4))

correction_Had  = -np.mean(HadCRU_t[idrHad])
correction_exp1 = -np.mean(exp1['T_ml'][idrmodel])
correction_exp2 = -np.mean(exp2['T_ml'][idrmodel])
correction_exp3 = -np.mean(exp3['T_ml'][idrmodel])

axes.plot(exp2['time'],exp2['T_ml']+correction_exp2,color='green',linewidth=1.0)
axes.plot(exp3['time'],exp3['T_ml']+correction_exp3,color='red',linewidth=1.0)

axes.plot(HadCRU_year,HadCRU_t+correction_Had,color='gray')

axes.plot(exp1['time'],exp1['T_ml']+correction_exp1,color='deepskyblue',linewidth=1.5)


id=np.where(exp1['time'] == 2010)
axes.scatter(exp1['time'][id],exp1['T_ml'][id]+correction_exp1, color='deepskyblue', zorder=5)

axes.text(1750, 1.55-0*0.18, 'Observed temperature 1850-2016, HadCRUT 4.5', va='top', color='gray')
axes.text(1750, 1.5-1*0.18, 'Two-layer model, commitment case d:', va='top', color=almost_black)
axes.text(1750, 1.5-2*0.18, r'ECS='+str(exp1['ECS'])+'K, $\epsilon$='+str(exp1['efficacy']), va='top', color='deepskyblue')
axes.text(1750, 1.5-3*0.18, r'ECS='+str(exp2['ECS'])+'K, $\epsilon$='+str(exp2['efficacy']), va='top', color='green')
axes.text(1750, 1.5-4*0.18, r'ECS='+str(exp3['ECS'])+'K, $\epsilon$='+str(exp3['efficacy']), va='top', color='red')


T_max = exp1['T_ml'][-1]+correction_exp1
axes.text(2100, T_max-0.06, str(round(T_max,2))+'K', ha='center', va='top', color='deepskyblue', fontsize=10)
T_max3 = exp3['T_ml'][-1]+correction_exp3
axes.text(2100, T_max3+0.05, str(round(T_max3,2))+'K', ha='center', va='bottom', color='red', fontsize=10)

id=np.where((exp1['time'] >= 2005) & (exp1['time'] <= 2015))
print(str(np.mean(exp1['T_ml'][id]+correction_exp1))+', '+ str(np.mean(exp1['imbalance'][id])))
print(str(np.mean(exp2['T_ml'][id]+correction_exp2))+', '+ str(np.mean(exp2['imbalance'][id])))
print(str(np.mean(exp3['T_ml'][id]+correction_exp3))+', '+ str(np.mean(exp3['imbalance'][id])))

axes.set_xlim(left=1750, right=2115)
axes.set_ylim(bottom=-1.5, top=1.5)
axes.yaxis.set_label_position("right")

axes.set_xlabel('Time (years)')
axes.set_ylabel('Temperature (K) relative to 1850-1899')

for ticks in axes.xaxis.get_ticklines() + axes.yaxis.get_ticklines():
    ticks.set_color(almost_black)

spines_to_remove        = ['top', 'left'] 
for spine in spines_to_remove:
    axes.spines[spine].set_visible(False)

axes.yaxis.tick_right()
axes.yaxis.set_ticks_position('right')
axes.xaxis.set_ticks_position('bottom')

axes.spines['bottom'].set_position('zero')

spines_to_keep = [ 'bottom', 'right']     
for spine in spines_to_keep:
    axes.spines[spine].set_linewidth(0.5)
    axes.spines[spine].set_color(almost_black)



plt.tight_layout()
plt.savefig('Figure_S1_two_layer_model_historical.pdf', dpi=300)
plt.close()
